open System_defs

(* On crée une fenêtre *)
let init () =
  Global.init (Format.sprintf "game_canvas:%dx%d:r=presentvsync" 800 600)


(* Question 1 *)
let init_walls () =
  let blue = Texture.color (Gfx.color 0 0 255 255) in
  let black = Texture.color (Gfx.color 0 0 0 255) in
  ignore (Box.create "wall_top" 0 0 800 40 blue infinity);
  ignore (Box.create "wall_bottom" 0 560 800 40 blue infinity);
  ignore (Box.create "wall_left" 0 40 40 520 black infinity);
  ignore (Box.create "wall_right" 760 40 40 520 black infinity)

(* Question 10 *)
let rock_resource = ref None 
let init_squares _dt =
  match !rock_resource with
    None -> true
    | Some r -> 
    let surface = Gfx.get_resource r in
    let w, h  = Gfx.surface_size surface in
    Gfx.debug "DIMENSIONS: %d, %d\n%!" w h;
    let () = Random.self_init () in
    let ctx = Gfx.get_context (Global.window ()) in
    for i = 0 to 9 do
      let x = 40 + Random.int 720 in
      let y = 40 + Random.int 500 in
      let mass = 1.0 +. Random.float 19.0 in
      let texture = (* Texture.image_from_surface ctx rock_surface 0 0 256 256 100 100 *)
        Texture.anim_from_surface ctx surface 32 256 256 100 100 (1+Random.int 10)
      in
      let s = Box.create (Printf.sprintf "square_%d" i ) x y 100 100 texture mass in
      s#sum_forces#set Vector.{ x = Random.float 0.25; y = Random.float 0.25 }
    done;
    false


  
let load_rock () =
  let ctx = Gfx.get_context (Global.window ()) in
  rock_resource := Some (Gfx.load_image ctx "resources/images/rock.png")
  
let wait_textures _dt =
  match !rock_resource with
      None -> failwith "Error"
    | Some r ->  not (Gfx.resource_ready r)
  
let update dt =
    Collision_system.update dt;
    Force_system.update dt;
    Move_system.update dt;
    Draw_system.update dt;
    ignore (Gfx.poll_event());
    true


let chain_functions l =
  let todo = ref l in
  (fun dt ->
    match !todo with
      [] -> false
    | f :: ll ->
      let res = f dt in
      if res then true
      else begin
        todo := ll;
        true
      end)

let run () =
  init ();
  init_walls ();
  load_rock ();
  Gfx.main_loop (chain_functions [ wait_textures; 
                                   init_squares;
                                   update ])
    




